// https://doc.rust-lang.org/stable/rust-by-example/std_misc/file/read_lines.html
use std::fs::File;
use std::io::{self, BufRead};
use std::path::Path;

pub fn read_lines<P>(filename: P) -> io::Result<io::Lines<io::BufReader<File>>>
where
    P: AsRef<Path>,
{
    let file = File::open(filename)?;
    Ok(io::BufReader::new(file).lines())
}

pub fn read_input_file<T>(input_path: &str, callback: T) -> String where T: Fn(Vec<String>) -> String {
    if let Ok(lines) = read_lines(&input_path) {
        let input_lines: Vec<String> = lines.map(|line| line.unwrap()).collect();
        callback(input_lines)
    } else {
        panic!("Failed to read lines from input file");
    }
}