use super::common::Spring;

pub fn part_one(input_lines: Vec<String>) -> String {

    let mut springs: Vec<Vec<Spring>> = Vec::new();
    let mut values: Vec<Vec<usize>> = Vec::new();

    for line in input_lines {

        let split_line: Vec<&str> = line.split(' ').collect();
        
        springs.push(parse_springs(split_line[0].to_string()));
        values.push(parse_nums(split_line[1].to_string()));
    }

    // for i in 0..springs.len() {
    //     println!("{:?} - {:?}", springs[i], values[i]);
    // }

    let mut res = 0;
    for i in 0..springs.len() {
        let search_spring = springs[i].clone();
        res += unknown_search(search_spring, values[i].clone());        
    }

    res.to_string()
}

fn parse_springs(line: String) -> Vec<Spring> {
    let mut spring_line: Vec<Spring> = Vec::new();

    for c in line.chars() {
        match c {
            '.' => spring_line.push(Spring::O),
            '#' => spring_line.push(Spring::D),
            _ => spring_line.push(Spring::U),
        }
    }

    spring_line
}

fn parse_nums(line: String) -> Vec<usize> {
    let nums: Vec<usize> = line.split(',').map(|x| x.parse().unwrap()).collect();
    nums
}

fn validate(spring_line: Vec<Spring>, values: Vec<usize>) -> usize {
    
    let mut validation_vals: Vec<usize> = Vec::new();
    let mut count: usize = 0;

    // println!("{:?}: {:?}", spring_line, values);

    for i in 0..spring_line.len() {
        match spring_line[i] {
            Spring::O => {
                if count > 0 {
                    validation_vals.push(count);
                    count = 0;
                }
            },
            Spring::D => {
                count += 1;
            },
            _ => {
                panic!("Invalid spring");
            },
        }
    }

    if count > 0 {
        validation_vals.push(count);
    }

    if validation_vals == values {
        return 1;
    }

    return 0;

}

fn unknown_search(spring_line: Vec<Spring>, values: Vec<usize>) -> usize {

    // println!("Unknown search: {:?}", spring_line);

    let mut res = 0;

    let unknowns: Vec<usize> = spring_line.iter().enumerate().filter(|(_, x)| **x == Spring::U).map(|(i, _)| i).collect();
    let i = unknowns.first();

    if i.is_none() {
        return validate(spring_line, values);
    }

    let i = i.unwrap();

    let mut op_spring_line = spring_line.clone();
    op_spring_line[*i] = Spring::O;
    res += unknown_search(op_spring_line, values.clone());

    let mut dmg_spring_line = spring_line.clone();
    dmg_spring_line[*i] = Spring::D;
    res += unknown_search(dmg_spring_line, values.clone());


    res
}